/*-
 * Copyright (c) 2005 Eugene Grosbein <eugen@grosbein.pp.ru>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the authors nor the names of contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char rcsid[] = "@(#)$Id: debug.c,v 1.1.1.1.2.2 2006/04/19 15:46:26 eugen Exp $";
#endif

/*
 * Debugging and training module.
 * Algorithms were not optimized due to the purpose.
 */

#include <sys/types.h>

#include <err.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sysexits.h>

#include "common.h"


typedef unsigned char	 stats_array[MAXBUF][8];
typedef stats_array	*stats_set[MAXBUF];

static stats_set 	*stats = NULL;
static unsigned		*tries = NULL;
static
volatile sig_atomic_t	 do_print = 0;

unsigned		max_interval = 0;

/*
 * Print memory area
 */
void
printbuf(const void *b, const ssize_t len)
{
  const unsigned char *buf = b;
  int i;

  for (i = 0; i < len; i++)
    fprintf(stderr, "%02x", ((unsigned)buf[i]) & 0xff);
  fputs(" ", stderr);
}

void sighandler(int sig) {
  do_print = 1;
}

void
print_stats(void) {
  stats_array	*bit;
  ssize_t	 i, len;
  unsigned	 t;
  unsigned char  b;

  if (!do_print)
    return;

  do_print = 0;
  if (stats == NULL) {
    fprintf(stderr, "no data yet\n");
    return;
  }
  
  for (len = 0; len < MAXBUF; len++) if ((bit = (*stats)[len]) != NULL) {
    fprintf(stderr,"time: %d\t\t\t\b\b\b\b\b", max_interval);
    for (i = 0, t = tries[len]; i < len; i++) {
      b = 0;
      if ((*bit)[i][0] * 2 > t) b += 1;
      if ((*bit)[i][1] * 2 > t) b += 2;
      if ((*bit)[i][2] * 2 > t) b += 4;
      if ((*bit)[i][3] * 2 > t) b += 8;
      if ((*bit)[i][4] * 2 > t) b += 16;
      if ((*bit)[i][5] * 2 > t) b += 32;
      if ((*bit)[i][6] * 2 > t) b += 64;
      if ((*bit)[i][7] * 2 > t) b += 128;
      fprintf(stderr,"%02x",(unsigned)b);
    }
    fprintf(stderr,"\n");
  }
}

void
update_stats(const void* buf, const ssize_t len) {
  const unsigned char	*b = buf;
  stats_array 		*bit;
  ssize_t 		 i;

  if (stats == NULL)
    if ((stats = calloc(1, sizeof(*stats))) == NULL)
      err(EX_OSERR,"calloc(1, %d)", sizeof(*stats));
  if (tries == NULL)
    if ((tries = calloc(MAXBUF, sizeof(*tries))) == NULL)
      err(EX_OSERR,"calloc(MAXBUF, %d)", sizeof(*tries));
  if ((*stats)[len] == NULL)
    if (((*stats)[len] = calloc(1, sizeof(stats_array))) == NULL)
      err(EX_OSERR,"calloc(1, %d)", sizeof(stats_array));
  
  if (tries[len] == 255)
    return;

  tries[len]++;
  for (i = 0, bit = (*stats)[len]; i < len; i++, b++) {
    (*bit)[i][0] += *b & 1 ;
    (*bit)[i][1] += (*b & 2) >> 1;
    (*bit)[i][2] += (*b & 4) >> 2;
    (*bit)[i][3] += (*b & 8) >> 3;
    (*bit)[i][4] += (*b & 16) >> 4;
    (*bit)[i][5] += (*b & 32) >> 5;
    (*bit)[i][6] += (*b & 64) >> 6;
    (*bit)[i][7] += (*b & 128) >> 7;
  }
}
